@@ -1,5 +1,6 @@
 from abc import abstractmethod, ABC
 from typing import Tuple, Optional, Callable, Any, List, Dict, Union
+from itertools import compress
 
 from scipy.stats import PearsonRConstantInputWarning
 from sklearn.preprocessing import StandardScaler
@@ -26,7 +27,6 @@ class Controller(ABC):
         self.__time_dataset: List[float] = []
         self.__error_dataset: List[float] = []
         self.__cost_dataset: List[float] = []
-        self.__is_call_exact: List[int] = []  # 1 if the exact model has been called
         self.__to_reshape = False
         self._exact: Callable[[List[float]], List[float]] = exact_fitness
         self._estimator: Estimator = estimator
@@ -38,9 +38,53 @@ class Controller(ABC):
             Path(self._controller_debug).open("w").close()
 
     @abstractmethod
-    def compute_objective(self, point: List[int]) -> List[float]:
+    def compute_objective(
+        self, point: List[int], decision_only: bool = False
+    ) -> List[float]:
         pass
 
+    def learn(
+        self,
+        point: List[float],
+        exec_time: float = None,
+        mab: Optional[Tuple[MabHandler, Union[int, float]]] = None,
+        mab_forced_probability: Optional[float] = None,
+        mab_forced_action: Optional[Union[int, float]] = None,
+        mab_weight: Optional[MabHandler] = None,
+        mab_weight_forced_probability: Optional[float] = None,
+        mab_weight_forced_action: Optional[Union[int, float]] = None,
+        is_point_in_context: bool = True,
+    ):
+        if mab:
+            if mab_weight:
+                time_weight = (
+                    mab_weight.predict(
+                        self._compute_weighting_context(self.get_mab().get_mean_cost())
+                    )
+                    / 100
+                )
+                mab[0].learn(
+                    self.get_time_error_z_score(
+                        exec_time, 0, *(time_weight, 1 - time_weight)
+                    ),
+                    self._compute_controller_context(point),
+                    mab_forced_probability,
+                ),
+                mab_weight.learn(
+                    self.get_time_error_correlation(),
+                    self._compute_weighting_context(self.get_mab().get_mean_cost()),
+                    mab_weight_forced_probability,
+                )
+            else:
+                mab[0].learn(
+                    self.get_time_error_z_score(
+                        exec_time,
+                        0,
+                    ),
+                    self._compute_controller_context(point),
+                    mab_forced_probability,
+                )
+
     def _compute_exact(
         self,
         point: List[int],
@@ -50,7 +94,7 @@ class Controller(ABC):
         mab_weight_forced_probability: Optional[int] = None,
         is_point_in_context: bool = True,
     ) -> Tuple[List[float], float]:
-        self.__is_call_exact.append(1)
+
         if self._self_exact:
             exact, exec_time = Controller.measure_execution_time(
                 self._exact, self._self_exact, point
@@ -58,57 +102,51 @@ class Controller(ABC):
         else:
             exact, exec_time = Controller.measure_execution_time(self._exact, point)
 
+        self.learn(
+            point=point,
+            exec_time=exec_time,
+            mab=mab,
+            mab_forced_probability=mab_forced_probability,
+            mab_weight=mab_weight,
+            mab_weight_forced_probability=mab_weight_forced_probability,
+            is_point_in_context=is_point_in_context,
+        )
+
         self._estimator.train(point, exact)
-        if mab:
-            if mab_weight:
-                time_weight = (
-                    mab_weight.predict(self._compute_weighting_context()) / 100
-                )
-                mab[0].learn(
-                    mab[1],
-                    self.get_time_error_z_score(
-                        exec_time, 0, *(time_weight, 1 - time_weight)
-                    ),
-                    self._compute_controller_context(
-                        point if is_point_in_context else None
-                    ),
-                    mab_forced_probability,
-                ),
-                mab_weight.learn(
-                    time_weight * 100,
-                    self.get_time_error_correlation(),
-                    self._compute_weighting_context(),
-                    mab_weight_forced_probability,
-                )
-            else:
-                mab[0].learn(
-                    mab[1],
-                    self.get_time_error_z_score(
-                        exec_time,
-                        0,
-                    ),
-                    self._compute_controller_context(
-                        point if is_point_in_context else None
-                    ),
-                    mab_forced_probability,
-                )
         return exact, exec_time
 
-    def _compute_controller_context(self, point: List[int] = None) -> List[float]:
-        if not point:
-            return [
-                self._estimator.get_error(),
-                np.mean(self.__time_dataset) if len(self.__time_dataset) > 0 else 0,
-            ]
-        else:
-            return [
-                self._estimator.get_error(),
-                np.mean(self.__time_dataset) if len(self.__time_dataset) > 0 else 0,
-            ] + point
+    @staticmethod
+    def __extract_time_context_features(series: List[float]):
+        return (
+            (np.mean(series), np.var(series), max(series), min(series), series[-1])
+            if len(series) > 0
+            else (0, 0, 0, 0, 0)
+        )
 
-    def _compute_weighting_context(self) -> List[float]:
-        estimation_calls = self.__is_call_exact.count(0)
-        return [estimation_calls, len(self.__is_call_exact) - estimation_calls]
+    @staticmethod
+    def _compute_controller_context(point: List[float]):
+        return point
+
+    # def _compute_controller_context(self, point: List[int] = None) -> List[float]:
+    #     exact_execution_times: List[float] = list(
+    #         compress(self.__time_dataset, self.__is_call_exact)
+    #     )
+    #     estimated_execution_times: List[float] = [
+    #         x for x in self.__time_dataset if x not in exact_execution_times
+    #     ]
+    #     context = [
+    #         self._estimator.get_error(),
+    #         *self.__extract_time_context_features(exact_execution_times),
+    #         *self.__extract_time_context_features(estimated_execution_times),
+    #     ]
+    #     if not point:
+    #         return context
+    #     else:
+    #         return context + point
+
+    @staticmethod
+    def _compute_weighting_context(mab_loss: float) -> List[float]:
+        return [mab_loss]
 
     def _compute_estimated(
         self,
@@ -117,43 +155,21 @@ class Controller(ABC):
         mab_weight: MabHandler = None,
         is_point_in_context: bool = True,
     ) -> Tuple[List[float], float]:
-        self.__is_call_exact.append(0)
+
         estimation, exec_time = Controller.measure_execution_time(
             self._estimator.predict,
             point,
         )
-        if mab:
-            if mab_weight:
-                time_weight = (
-                    mab_weight.predict(self._compute_weighting_context()) / 100
-                )
-                mab[0].learn(
-                    mab[1],
-                    self.get_time_error_z_score(
-                        exec_time,
-                        self._estimator.get_error(),
-                        *(time_weight, 1 - time_weight)
-                    ),
-                    self._compute_controller_context(
-                        point if is_point_in_context else None
-                    ),
-                )
-                mab_weight.learn(
-                    time_weight,
-                    self.get_time_error_correlation(),
-                    self._compute_weighting_context(),
-                )
-            else:
-                mab[0].learn(
-                    mab[1],
-                    self.get_time_error_z_score(
-                        exec_time,
-                        self._estimator.get_error(),
-                    ),
-                    self._compute_controller_context(
-                        point if is_point_in_context else None
-                    ),
-                )
+
+        self.learn(
+            point,
+            exec_time,
+            mab,
+            None,
+            mab_weight,
+            None,
+            is_point_in_context,
+        )
 
         return estimation, exec_time
 
@@ -241,3 +257,15 @@ class Controller(ABC):
         out = func(*args, **kwargs)
         execution_time = time.time() - start
         return out, execution_time
+
+    @abstractmethod
+    def get_mab(self) -> MabHandler:
+        pass
+
+    @abstractmethod
+    def get_weight_mab(self) -> MabHandler:
+        pass
+
+    @abstractmethod
+    def get_parameters(self):
+        pass
