@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import List, Callable, Dict, Any, Optional
+from typing import List, Callable, Dict, Any, Optional, Union
 
 from movado.controller import Controller
 from movado.estimator import Estimator
@@ -16,7 +16,6 @@ class MabController(Controller):
         debug: bool = False,
         skip_debug_initialization=False,
         cover: int = 3,
-        epsilon: float = 0.2,
         mab_weight: bool = True,
         mab_weight_epsilon: float = 0.2,
         mab_weight_bandwidth: int = 1,
@@ -27,16 +26,29 @@ class MabController(Controller):
             self_exact=self_exact,
             debug=debug,
         )
+        self.__params = (
+            "Model_Parameters",
+            {
+                "cover": cover,
+                "mab_weight_epsilon": mab_weight_epsilon,
+                "mab_weight_bandwidth": mab_weight_bandwidth,
+            },
+        )
 
-        self.__epsilon = epsilon
         self.__cover = cover
-        self.__mab = MabHandlerCB(arms=2, debug=debug, cover=3)
+        self.__mab = MabHandlerCB(
+            arms=2,
+            debug=debug,
+            cover=3,
+            controller_params={self.__params[0]: self.__params[1]},
+        )
         self.__weight_mab = None
         if mab_weight:
             self.__weight_mab = MabHandlerCATS(
                 debug=debug,
                 epsilon=mab_weight_epsilon,
                 bandwidth=mab_weight_bandwidth,
+                controller_params={self.__params[0]: self.__params[1]},
                 debug_path="mab_weight",
             )
         self.__is_first_call = True
@@ -49,9 +61,13 @@ class MabController(Controller):
             "Model_Parameters, Point, Exec_Time, Error, Estimation\n"
         )
 
-    def compute_objective(self, point: List[int]) -> List[float]:
+    def compute_objective(
+        self, point: List[int], decision_only: bool = False
+    ) -> Union[List[float], int]:
         decision = self.__mab.predict(self._compute_controller_context(point))
         accuracy = self._estimator.get_error()
+        if decision_only:
+            return decision - 1
         if decision == 2 or accuracy == 0.0:
             out, exec_time = self._compute_exact(
                 point,
@@ -98,3 +114,12 @@ class MabController(Controller):
 
     def get_mean_cost(self):
         return self.__mab.get_mean_cost()
+
+    def get_mab(self) -> MabHandlerCB:
+        return self.__mab
+
+    def get_weight_mab(self) -> MabHandlerCATS:
+        return self.__weight_mab
+
+    def get_parameters(self):
+        return self.__params
