@@ -11,9 +11,16 @@ class MabHandlerCATS(MabHandler):
         bandwidth: int = 1,
         epsilon: float = 0.2,
         debug=False,
+        controller_params: dict = None,
         debug_path: str = "mab",
+        skip_debug_initialization: bool = False,
     ):
-        super(MabHandlerCATS, self).__init__(debug, debug_path)
+        super(MabHandlerCATS, self).__init__(
+            debug,
+            debug_path,
+            controller_params=controller_params,
+            skip_debug_initialization=skip_debug_initialization,
+        )
         self._sample_prefix = "ca "
         if bandwidth < 0:
             raise Exception(
@@ -31,7 +38,7 @@ class MabHandlerCATS(MabHandler):
             + str(mab_actions)
             + "  --bandwidth "
             + str(bandwidth)
-            + " --min_value 0 --max_value 100 --chain_hash --coin --epsilon 0.2 -q ::"
+            + " --min_value 0 --max_value 100 --chain_hash --coin --epsilon 0.2 --quiet"
         )
 
     def predict(self, context: List[float]) -> float:
@@ -40,5 +47,6 @@ class MabHandlerCATS(MabHandler):
             context_str += str(feature) + " "
         context_str.strip()
         action: Tuple[float, float] = self._mab.predict(context_str)
+        self._last_action = action[0]
         self._last_predict_probability = action[1]
         return action[0]
