@@ -13,11 +13,17 @@ class MabHandlerCB(MabHandler):
         cover: float = 3,
         controller_params: dict = None,
         debug_path: str = "mab",
+        skip_debug_initialization: bool = False,
     ):
         super().__init__(
-            debug, controller_params=controller_params, debug_path=debug_path
+            debug,
+            controller_params=controller_params,
+            debug_path=debug_path,
+            skip_debug_initialization=skip_debug_initialization,
+        )
+        self._mab = pyvw.vw(
+            "--cb_explore " + str(arms) + " --cover " + str(cover) + " --quiet"
         )
-        self._mab = pyvw.vw("--cb_explore " + str(arms) + " --cover " + str(cover))
 
     def predict(self, context: List[float]) -> int:
         context_str: str = "| "
@@ -28,6 +34,7 @@ class MabHandlerCB(MabHandler):
             self._mab.predict(context_str)
         )
         self._last_predict_probability = prediction[1]
+        self._last_action = prediction[0]
         return prediction[0] + 1
 
     @staticmethod
