@@ -1,5 +1,5 @@
 from collections import OrderedDict
-from typing import List, Callable, Optional
+from typing import List, Callable
 
 from movado.controller import Controller
 from movado.estimator import Estimator
@@ -15,7 +15,7 @@ import movado.movado_static
 from movado.mab_controller import MabController  # pylint: disable=unused-import
 
 # noinspection PyUnresolvedReferences
-from movado.meta_controller import MetaController  # pylint: disable=unused-import
+from movado.voting_controller import VotingController  # pylint: disable=unused-import
 
 # noinspection PyUnresolvedReferences
 from movado.kernel_regression_model import (
@@ -82,10 +82,30 @@ def approximate(
                     if not selected_controller
                     else selected_controller
                 )
-
-                controller = controller(
-                    func,
+                if selected_controller is MabController:
+                    params = OrderedDict(
+                        {
+                            "cover": [3, 5, 7, 11, 20],
+                            "mab_weight_epsilon": [0.1, 0.2, 0.3, 0.4],
+                            "mab_weight_bandwidth": [1, 5, 10, 20, 50, 75],
+                        }
+                    )
+                else:
+                    # All other cases are handled as a DistanceController
+                    params = OrderedDict(
+                        {
+                            "nth_nearest": [1, 3, 5, 7],
+                            "mab_epsilon": [0.1, 0.2, 0.3],
+                            "mab_bandwidth": [1, 5, 10, 20],
+                            "mab_weight_epsilon": [0.1, 0.2, 0.3],
+                            "mab_weight_bandwidth": [1, 5, 10, 20],
+                        }
+                    )
+                controller = VotingController(
+                    controller,
                     estimator,
+                    func,
+                    params,
                     self_exact=None if len(wrapper_args) == 1 else wrapper_args[0],
                     **{
                         k[k.find("_") + 1 :]: v
